package cn.iocoder.yudao.module.basic.service.inputsinfo;

import cn.iocoder.yudao.framework.common.enums.CodeEnum;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.basic.api.redis.RedisBasicInfoUtilApi;
import cn.iocoder.yudao.module.basic.controller.admin.inputsinfo.vo.*;
import cn.iocoder.yudao.module.basic.convert.inputsinfo.InputsInfoConvert;
import cn.iocoder.yudao.module.basic.dal.dataobject.inputsinfo.InputsInfoDO;
import cn.iocoder.yudao.module.basic.dal.dataobject.unit.UnitDO;
import cn.iocoder.yudao.module.basic.dal.mysql.inputsinfo.InputsInfoMapper;
import cn.iocoder.yudao.module.basic.dal.mysql.unit.UnitMapper;
import cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.basic.service.businessinfo.BusinessInfoServiceImpl;
import cn.iocoder.yudao.module.product.api.inputsbatch.InputsBatchApi;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.basic.enums.ErrorCodeConstants.*;

/**
 * 投入品 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class InputsInfoServiceImpl implements InputsInfoService {

    @Resource
    private InputsInfoMapper inputsInfoMapper;

    @Resource
    private RedisBasicInfoUtilApi redisUtil;

    @Resource
    private BusinessInfoServiceImpl businessInfoService;

    @Resource
    private InputsBatchApi inputsBatchApi;

    @Resource
    private UnitMapper unitMapper;

    /**
     * 创建投入品
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    @Override
    public Long createInputsInfo(InputsInfoCreateReqVO createReqVO) {
        //校验所属商户是否存在
        businessInfoService.validateBusinessInfoExists(createReqVO.getBelongBusinessId());
        //校验投入品名称是否重复
        validateInputsInfoExists((long)0,createReqVO.getInputsName(),createReqVO.getBelongBusinessId());
        // 插入
        InputsInfoDO inputsInfo = InputsInfoConvert.INSTANCE.convert(createReqVO);
        try {
            inputsInfo.setInputsId(CodeEnum.INPUTS_INFO_CODE.getCode());
            // 设置商品规格
            if (inputsInfo.getIsBaseUnit()) {
                inputsInfo.setInputsSpecs("kg");
                // 设置默认单位ID为0
                inputsInfo.setInputsUnitId(0L);
            } else {
                BigDecimal conversionValue = inputsInfo.getConversionValue();
                if (conversionValue != null) {
                    String inputsSpecs = String.format("%skg/%s", conversionValue, getUnitName(inputsInfo.getInputsUnitId()));
                    inputsInfo.setInputsSpecs(inputsSpecs);
                }
            }
            inputsInfoMapper.insert(inputsInfo);
        } catch(Exception e){
            e.printStackTrace();
            throw exception(INPUTS_INFO_FAIL_CREATE);
        }

        redisUtil.updateInputsInfoRedis(inputsInfo.getId(),InputsInfoConvert.INSTANCE.convertToDTO(inputsInfo));
        // 返回
        return inputsInfo.getId();
    }

    // 根据商品单位ID获取单位名称
    private String getUnitName(Long unitId) {
        UnitDO unit = unitMapper.selectById(unitId);
        if (unit != null) {
            return unit.getUnitName();
        } else {
            throw new RuntimeException("无法找到对应的单位");
        }
    }
    /**
     * 更新投入品
     *
     * @param updateReqVO 更新信息
     */
    @Override
    public void updateInputsInfo(InputsInfoUpdateReqVO updateReqVO) {
        // 校验存在以及校验投入品名称是否重复
        validateInputsInfoExists(updateReqVO.getId(),updateReqVO.getInputsName(),updateReqVO.getBelongBusinessId());
        //校验所属商户是否存在
        businessInfoService.validateBusinessInfoExists(updateReqVO.getBelongBusinessId());
        //获取原始投入品信息
        InputsInfoDO inputsInfo = inputsInfoMapper.selectById(updateReqVO.getId());
        if (inputsInfo == null){
            throw exception(INPUTS_INFO_NOT_EXISTS);
        }
        // 更新
        InputsInfoDO updateObj = InputsInfoConvert.INSTANCE.convert(updateReqVO);
        // 设置商品规格
        if (updateObj.getIsBaseUnit()) {
            updateObj.setInputsSpecs("kg");
            updateObj.setInputsUnitId(0L);
        } else {
            String InputsSpecs = String.format("%skg/%s", updateObj.getConversionValue(), getUnitName(updateObj.getInputsUnitId()));
            updateObj.setInputsSpecs(InputsSpecs);
        }
        try {
            inputsInfoMapper.updateById(updateObj);
        } catch (Exception e) {
            e.printStackTrace();
            throw exception(INPUTS_INFO_FAIL_UPDATE);
        }
        redisUtil.updateInputsInfoRedis(updateObj.getId(),InputsInfoConvert.INSTANCE.convertToDTO(updateObj));
    }

    /**
     * 删除投入品
     *
     * @param id 编号
     */
    @Override
    public void deleteInputsInfo(Long id) {
        // 校验存在
        validateInputsInfoExists(id,null,(long)0);
        //校验投入品是否存在关联投入品批次
        if(!inputsBatchApi.getInputsBatchListByInputId(inputsInfoMapper.selectById(id).getInputsId()).getData().isEmpty()){
            throw  exception(INPUTS_INFO_FAIL_DEL_BELONG_INPUTS_BATCH_EXISTS);
        }
        // 删除
        try{
            inputsInfoMapper.deleteById(id);
        }catch (Exception e){
            e.printStackTrace();
            throw exception(INPUTS_INFO_FAIL_DEL);
        }

        redisUtil.delInputsInfoRedis(id);
    }

    private void validateInputsInfoExists(Long id,String inputsName,Long belongBusinessId) {
        if (!id.equals((long)0) && inputsInfoMapper.selectById(id) == null) {
            throw exception(ErrorCodeConstants.INPUTS_INFO_NOT_EXISTS);
        }

        if(StringUtils.hasText(inputsName) && !belongBusinessId.equals((long)0) &&
                !inputsInfoMapper.selectList(new LambdaQueryWrapperX<InputsInfoDO>()
                .eq(InputsInfoDO::getInputsName,inputsName).eq(InputsInfoDO::getBelongBusinessId,belongBusinessId)
                .ne(InputsInfoDO::getId,id)).isEmpty()){
            throw exception(ErrorCodeConstants.INPUTS_INFO_NAME_IS_EXISTS);
        }
    }

    @Override
    public InputsInfoDO getInputsInfo(Long id) {
        return inputsInfoMapper.selectById(id);
    }

    /**
     * 获得投入品分页
     *
     * @param pageReqVO 分页查询
     * @return 投入品分页
     */
    @Override
    public PageResult<InputsInfoDO> getInputsInfoPage(InputsInfoPageReqVO pageReqVO) {
        Page<InputsInfoDO> page = new Page<>(pageReqVO.getPageNo(),pageReqVO.getPageSize());
        IPage<InputsInfoDO> inputsInfoDOIPage = inputsInfoMapper.selectInputsPageByInfo(page, pageReqVO.getInputsName(),
                pageReqVO.getInputsId(), pageReqVO.getInputsUnitId(), pageReqVO.getBelongBusinessName());
        // 根据InputsUnitId设置InputsUnitName
        List<InputsInfoDO> records = inputsInfoDOIPage.getRecords();
        for (InputsInfoDO inputsInfoDO : records) {
            if (inputsInfoDO.getInputsUnitId() == 0) {
                inputsInfoDO.setInputsUnitName("kg");
            } else {
                inputsInfoDO.setInputsUnitName(getUnitName(inputsInfoDO.getInputsUnitId()));
            }
        }
        PageResult<InputsInfoDO> inputsInfoDOPageResult = new PageResult<InputsInfoDO>().setList(records);
        inputsInfoDOPageResult.setTotal(inputsInfoDOIPage.getTotal());
        return inputsInfoDOPageResult;
    }

    /**
     * 获得投入品列表
     *
     * @param exportReqVO 查询条件
     * @return 投入品列表
     */
    @Override
    public List<InputsInfoDO> getInputsInfoList(InputsInfoExportReqVO exportReqVO) {
        return inputsInfoMapper.selectList(exportReqVO);
    }

    /**
     * 获取所有投入品
     *
     * @return
     */
    @Override
    public List<InputsInfoDO> getInputsInfoInputsInfoList() {
        LambdaQueryWrapper<InputsInfoDO> wrapper = new LambdaQueryWrapper<>();
        return inputsInfoMapper.selectList(wrapper);
    }

    /**
     * 根据指定的商户id 获取所有名下多个投入品
     *
     * @param belongBusinessId  商户id列表
     * @return 返回投入品列表
     */
    @Override
    public List<InputsInfoDO> getInputsInfoListByBelongBusinessId(List<Long> belongBusinessId) {
        LambdaQueryWrapper<InputsInfoDO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(InputsInfoDO::getBelongBusinessId, belongBusinessId);
        return inputsInfoMapper.selectList(queryWrapper);
    }

    @Override
    public List<InputsInfoRespVO> getInputsInfoByBusinessInfoId(Long businessInfoId) {
        return inputsInfoMapper.getInputsInfoByBusinessInfoId(businessInfoId);
    }

    @Override
    public List<InputsInfoForTransferRespVO> queryListOfMiniResult(Long businessInfoId) {
        return inputsInfoMapper.queryListOfMiniResult(businessInfoId);
    }
}
