package cn.iocoder.yudao.module.product.controller.admin.landinfo;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.LandInfoCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.LandInfoPageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.LandInfoRespVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.LandInfoUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landinfo.LandInfoConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landinfo.LandInfoDO;
import cn.iocoder.yudao.module.product.service.landinfo.LandInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "管理后台 - 农事土地信息管理")
@RestController
@RequestMapping("/product/land-info")
@Validated
public class LandInfoController {

    @Resource
    private LandInfoService landInfoService;

    @PostMapping("/create")
    @ApiOperation("创建农事土地信息")
    @PreAuthorize("@ss.hasPermission('product:land-info:create')")
    public CommonResult<Long> createLandInfo(@Valid @RequestBody LandInfoCreateReqVO createReqVO) {
        return success(landInfoService.createLandInfo(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新农事土地信息")
    @PreAuthorize("@ss.hasPermission('product:land-info:update')")
    public CommonResult<Boolean> updateLandInfo(@Valid @RequestBody LandInfoUpdateReqVO updateReqVO) {
        landInfoService.updateLandInfo(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除农事土地信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-info:delete')")
    public CommonResult<Boolean> deleteLandInfo(@RequestParam("id") Long id) {
        landInfoService.deleteLandInfo(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得农事土地信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-info:query')")
    public CommonResult<LandInfoRespVO> getLandInfo(@RequestParam("id") Long id) {
        LandInfoDO landInfo = landInfoService.getLandInfo(id);
        return success(LandInfoConvert.INSTANCE.convert(landInfo));
    }

    @GetMapping("/list")
    @ApiOperation("获得农事土地信息列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('product:land-info:query')")
    public CommonResult<List<LandInfoRespVO>> getLandInfoList(@RequestParam("ids") Collection<Long> ids) {
        List<LandInfoDO> list = landInfoService.getLandInfoList(ids);
        return success(LandInfoConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得农事土地信息分页")
    @PreAuthorize("@ss.hasPermission('product:land-info:query')")
    public CommonResult<PageResult<LandInfoRespVO>> getLandInfoPage(@Valid LandInfoPageReqVO pageVO) {
        PageResult<LandInfoDO> pageResult = landInfoService.getLandInfoPage(pageVO);
        return success(LandInfoConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/nameList")
    @ApiOperation("获得所有地块名称")
    @PreAuthorize("@ss.hasPermission('product:land-info:query')")
    public CommonResult<List<LandInfoRespVO>> getLandInfoLandNameList() {
        List<LandInfoDO> nameList = landInfoService.getLandInfoLandNameList();
        return success(LandInfoConvert.INSTANCE.convertList(nameList));
    }

    @GetMapping("/getByBusinessId")
    @ApiOperation("根据商户ID获得地块信息")
    @PreAuthorize("@ss.hasPermission('product:land-info:query')")
    @ApiImplicitParam(name = "belongBusinessId", value = "所属商户id", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<LandInfoDO>> getBusinessNamesByLandId(@RequestParam("belongBusinessId") Long belongBusinessId) {
        List<LandInfoDO> namesByLandId = landInfoService.getLandInfoByBelongBusinessId(belongBusinessId);
        return success(namesByLandId);
    }

}
