package cn.iocoder.yudao.module.product.controller.admin.landjob;

import cn.iocoder.yudao.framework.common.enums.BlockStateEnum;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.product.controller.admin.landjob.vo.*;
import cn.iocoder.yudao.module.product.convert.landjob.LandJobConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landjob.LandJobDO;
import cn.iocoder.yudao.module.product.service.landjob.LandJobService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Api(tags = "管理后台 - 种植作业")
@RestController
@RequestMapping("/product/land-job")
@Validated
public class LandJobController {

    @Resource
    private LandJobService landJobService;

    @PostMapping("/create")
    @ApiOperation("创建种植/养殖/畜牧作业")
    @PreAuthorize("@ss.hasPermission('product:land-job:create')")
    public CommonResult<Long> createLandJob(@Valid @RequestBody LandJobCreateReqVO createReqVO) {
        return success(landJobService.createLandJob(createReqVO));
    }


    @PutMapping("/update")
    @ApiOperation("更新种植/养殖/畜牧作业")
    @PreAuthorize("@ss.hasPermission('product:land-job:update')")
    public CommonResult<Boolean> updateLandJob(@Valid @RequestBody LandJobUpdateReqVO updateReqVO) {
        landJobService.updateLandJob(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除种植/养殖/畜牧作业")
    @ApiImplicitParam(name = "id", value = "编号", required = true, dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-job:delete')")
    public CommonResult<Boolean> deleteLandJob(@RequestParam("id") Long id) {
        landJobService.deleteLandJob(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得种植/养殖/畜牧作业")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:land-job:query')")
    public CommonResult<LandJobRespVO> getLandJob(@RequestParam("id") Long id) {
        LandJobRespVO landJob = landJobService.getLandJob(id);
        return success(landJob);
    }

    @GetMapping("/list")
    @ApiOperation("获得种植/养殖/畜牧作业列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('product:land-job:query')")
    public CommonResult<List<LandJobRespVO>> getLandJobList(@RequestParam("ids") Collection<Long> ids) {
        List<LandJobDO> list = landJobService.getLandJobList(ids);
        return success(LandJobConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得种植/养殖/畜牧作业分页")
    @PreAuthorize("@ss.hasPermission('product:land-job:query')")
    public CommonResult<PageResult<LandJobRespVO>> getLandJobPage(@Valid LandJobPageReqVO pageVO) {
        PageResult<LandJobDO> pageResult = landJobService.getLandJobPage(pageVO);
        return success(LandJobConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出种植/养殖/畜牧作业 Excel")
    @PreAuthorize("@ss.hasPermission('product:land-job:export')")
    @OperateLog(type = EXPORT)
    public void exportLandJobExcel(@Valid LandJobExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<LandJobDO> list = landJobService.getLandJobList(exportReqVO);
        // 导出 Excel
        List<LandJobExcelVO> datas = LandJobConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "种植/养殖/畜牧作业.xls", "数据", LandJobExcelVO.class, datas);
    }

    @PutMapping("/upChain")
    @ApiOperation("上链")
    @ApiImplicitParam(name = "id", value = "id", required = true, example = "1", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:harvest-info:update')")
    public CommonResult<Boolean> upChain(@RequestParam("id") Long id) {
        landJobService.upChain(id);
        return CommonResult.success(true);
    }

    @PutMapping("/freeze")
    @ApiOperation("冻结")
    @ApiImplicitParam(name = "id", value = "ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inspection:freeze')")
    public CommonResult<Boolean> freeze(@RequestParam("id") Long id) {
        landJobService.updateChainStatus(id, BlockStateEnum.FREEZE.getBlockState());
        return success(Boolean.TRUE);
    }

    @PutMapping("/active")
    @ApiOperation("激活")
    @ApiImplicitParam(name = "id", value = "ID", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('product:inspection:active')")
    public CommonResult<Boolean> active(@RequestParam("id") Long id) {
        landJobService.updateChainStatus(id, BlockStateEnum.ACTIVATION.getBlockState());
        return success(Boolean.TRUE);
    }

}
