package cn.iocoder.yudao.module.product.service.inputsbatch;

import cn.iocoder.yudao.framework.common.enums.BlockStateEnum;
import cn.iocoder.yudao.framework.common.enums.CodeEnum;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.chain.api.blockcertificate.BlockCertificateApi;
import cn.iocoder.yudao.module.chain.api.blockcertificate.dto.UpdateStatusReqDTO;
import cn.iocoder.yudao.module.chain.enums.ChainStatusEnum;
import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.InputsBatchCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.InputsBatchExportReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.InputsBatchPageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.inputsbatch.vo.InputsBatchSaveReqVO;
import cn.iocoder.yudao.module.product.convert.inputsbatch.InputsBatchConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.inputsbatch.InputsBatchDO;
import cn.iocoder.yudao.module.product.dal.mysql.inputsbatch.InputsBatchMapper;
import cn.iocoder.yudao.module.product.enums.ErrorCodeConstants;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 投入品批次 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class InputsBatchServiceImpl implements InputsBatchService {

    @Resource
    private InputsBatchMapper inputsBatchMapper;
    @Resource
    private BlockCertificateApi blockCertificateApi;

    @Override
    public Long createInputsBatch(InputsBatchCreateReqVO createReqVO) {
        // 插入
        InputsBatchDO inputsBatch = InputsBatchConvert.INSTANCE.convert(createReqVO);
        inputsBatchMapper.insert(inputsBatch);
        // 返回
        return inputsBatch.getId();
    }

    @Override
    public void updateInputsBatch(InputsBatchSaveReqVO updateReqVO) {
        // 校验存在
        validateInputsBatchExists(updateReqVO.getId());
        // 更新
        InputsBatchDO updateObj = InputsBatchConvert.INSTANCE.convert(updateReqVO);
        inputsBatchMapper.updateById(updateObj);
    }

    @Override
    public void deleteInputsBatch(Long id) {
        // 校验存在
        validateInputsBatchExists(id);
        // 删除
        inputsBatchMapper.deleteById(id);
    }

    private void validateInputsBatchExists(Long id) {
        if (inputsBatchMapper.selectById(id) == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUTS_BATCH_NOT_EXISTS);
        }
    }

    @Override
    public InputsBatchDO getInputsBatch(Long id) {
        return inputsBatchMapper.selectById(id);
    }

    @Override
    public List<InputsBatchDO> getInputsBatchList(Collection<Long> ids) {
        return inputsBatchMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<InputsBatchDO> getInputsBatchPage(InputsBatchPageReqVO pageReqVO) {
        // pageReqVO.setBlockState(BlockStateEnum.ACTIVATION.getBlockState());
        return inputsBatchMapper.selectPage(pageReqVO);
    }

    @Override
    public List<InputsBatchDO> getInputsBatchList(InputsBatchExportReqVO exportReqVO) {
        return inputsBatchMapper.selectList(exportReqVO);
    }

    @Override
    public List<String> getInputsBatchNumberList(Integer count) {
        List<String> results = new ArrayList<>();
        for (int i = 0; i < count; i++) {
            results.add(CodeEnum.INPUTS_BATCH_CODE.getCode());
        }
        return results;
    }

    @Override
    public void deleteBatch(List<Long> ids) {
        List<InputsBatchDO> batchs = inputsBatchMapper.selectList(new LambdaQueryWrapperX<InputsBatchDO>().in(InputsBatchDO::getId, ids));
        if(batchs.size() != ids.size()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUTS_BATCH_NOT_EXISTS);
        }
        inputsBatchMapper.deleteBatchIds(ids);
    }

    /**
     * 根据投入品id获取所有投入品批次
     *
     * @param inputsId
     * @return
     */
    @Override
    @Transactional
    public List<String> getInputsBatchInputsBatchIdList(String inputsId) {
        return inputsBatchMapper.selectList(new QueryWrapper<InputsBatchDO>()
                //表示查询字段inputs_id等于给定的 inputsId值的记录。
                .eq("inputs_id",inputsId))
                .stream()
                //将查询结果中的每个InputsBatchDO对象映射为其 inputsBatchId属性的值。
                .map(InputsBatchDO::getInputsBatchId)
                .distinct()  //去除重复的批次ID。
                .collect(Collectors.toList()); // 将结果收集为一个列表
    }

    @Override
    @Transactional
    public void updateChainStatus(Long id, Integer state) {
        InputsBatchDO inputsBatchDO = inputsBatchMapper.selectById(id);
        if(inputsBatchDO == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUTS_BATCH_NOT_EXISTS);
        }
        if(inputsBatchDO.getBlockState().equals(BlockStateEnum.NOT_ON_CHAIN.getBlockState())) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.INPUT_BATCH_NOT_ACTIVATION);
        }

        List<UpdateStatusReqDTO> batchsReqDTOS = new ArrayList<>();
        UpdateStatusReqDTO harvestReqDTO = new UpdateStatusReqDTO();
        harvestReqDTO.setChainOperateId(inputsBatchDO.getBlockChainId());
        harvestReqDTO.setChainStatus(state.equals(BlockStateEnum.ACTIVATION.getBlockState()) ? ChainStatusEnum.ACTIVE.getStatus() : ChainStatusEnum.FREEZE.getStatus());
        batchsReqDTOS.add(harvestReqDTO);

        if(blockCertificateApi.updateStatusBatch(batchsReqDTOS).isError()) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.UPDATE_CREATE_BLOCK_CERTIFICATE_FAIL);
        }

        InputsBatchDO inputsBatch = new InputsBatchDO();
        inputsBatch.setId(id);
        inputsBatch.setBlockState(state.equals(BlockStateEnum.ACTIVATION.getBlockState()) ? BlockStateEnum.ACTIVATION.getBlockState() : BlockStateEnum.FREEZE.getBlockState());
        inputsBatchMapper.updateById(inputsBatch);
    }

}
