package cn.iocoder.yudao.module.product.service.landinfo;

import cn.iocoder.yudao.framework.common.enums.CodeEnum;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.LandInfoCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.LandInfoPageReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landinfo.vo.LandInfoUpdateReqVO;
import cn.iocoder.yudao.module.product.convert.landinfo.LandInfoConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landinfo.LandInfoDO;
import cn.iocoder.yudao.module.product.dal.mysql.landinfo.LandInfoMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_INFO_NOT_EXISTS;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_INFO_NOT_LESS_THAN_ZERO;

/**
 * 农事土地信息 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class LandInfoServiceImpl implements LandInfoService {

    @Resource
    private LandInfoMapper landInfoMapper;

    /**
     * 创建农事土地信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    @Override
    public Long createLandInfo(LandInfoCreateReqVO createReqVO) {
        // 校验size
        BigDecimal size = createReqVO.getSize();
        if (size != null && size.compareTo(BigDecimal.ZERO) < 0) {
            throw exception(LAND_INFO_NOT_LESS_THAN_ZERO);
        }
        // 插入
        LandInfoDO landInfo = LandInfoConvert.INSTANCE.convert(createReqVO);
        landInfo.setLandId(CodeEnum.LAND_INFO_CODE.getCode());
        landInfoMapper.insert(landInfo);
        // 返回
        return landInfo.getId();
    }

    /**
     * 更新农事土地信息
     *
     * @param updateReqVO 更新信息
     */
    @Override
    public void updateLandInfo(LandInfoUpdateReqVO updateReqVO) {
        // 校验存在
        validateLandInfoExists(updateReqVO.getId());
        // 校验size
        BigDecimal size = updateReqVO.getSize();
        if (size != null && size.compareTo(BigDecimal.ZERO) < 0) {
            throw exception(LAND_INFO_NOT_LESS_THAN_ZERO);
        }
        // 更新
        LandInfoDO updateObj = LandInfoConvert.INSTANCE.convert(updateReqVO);
        landInfoMapper.updateById(updateObj);
    }

    /**
     * 删除农事土地信息
     *
     * @param id 编号
     */
    @Override
    public void deleteLandInfo(Long id) {
        // 校验存在
        validateLandInfoExists(id);
        // 删除
        landInfoMapper.deleteById(id);
    }

    private void validateLandInfoExists(Long id) {
        if (landInfoMapper.selectById(id) == null) {
            throw exception(LAND_INFO_NOT_EXISTS);
        }
    }

    /**
     * 获得农事土地信息
     *
     * @param id 编号
     * @return 农事土地信息
     */
    @Override
    public LandInfoDO getLandInfo(Long id) {
        return landInfoMapper.selectById(id);
    }

    @Override
    public LandInfoDO getLandInfoByLandId(String landId) {
        return landInfoMapper.selectOne(new LambdaQueryWrapperX<LandInfoDO>().eq(LandInfoDO::getLandId, landId));
    }


    /**
     * 获得农事土地信息列表
     *
     * @param ids 编号
     * @return 农事土地信息列表
     */
    @Override
    public List<LandInfoDO> getLandInfoList(Collection<Long> ids) {
        return landInfoMapper.selectBatchIds(ids);
    }

    /**
     * 获得农事土地信息分页
     *
     * @param pageReqVO 分页查询
     * @return 农事土地信息分页
     */
    @Override
    public PageResult<LandInfoDO> getLandInfoPage(LandInfoPageReqVO pageReqVO) {
        Page<LandInfoDO> page = new Page<>(pageReqVO.getPageNo(),pageReqVO.getPageSize());
        IPage<LandInfoDO> landJobDOIPage = landInfoMapper.selectLandInfoById(page,pageReqVO.getLandName(),
                pageReqVO.getLandId(),pageReqVO.getBelongBusinessName(),pageReqVO.getState());
        PageResult<LandInfoDO> landInfoDOPageResult = new PageResult<LandInfoDO>().setList(landJobDOIPage.getRecords());
        landInfoDOPageResult.setTotal(landJobDOIPage.getTotal());
        return landInfoDOPageResult;
    }

    /**
     * 获取所有地块昵称
     *
     * @return
     */
    @Override
    public List<LandInfoDO> getLandInfoLandNameList() {
        //查询所有地块名称
        LambdaQueryWrapper<LandInfoDO> wrapper = new LambdaQueryWrapper<>();
        return landInfoMapper.selectList(wrapper);
    }

    @Override
    public List<LandInfoDO> getLandInfoByBelongBusinessId(Long belongBusinessId) {
        return landInfoMapper.selectList(new QueryWrapper<LandInfoDO>()
                .eq("belong_business_id", belongBusinessId));
    }
}


