package cn.iocoder.yudao.module.product.service.landplan;

import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.common.enums.CodeEnum;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.chain.enums.OperateDetailTypeEnum;
import cn.iocoder.yudao.module.chain.enums.OperateTypeEnum;
import cn.iocoder.yudao.module.product.controller.admin.harvestbatch.vo.BlockCertificateSaveReqVO;
import cn.iocoder.yudao.module.product.controller.admin.harvestbatch.vo.FarmingPlantReqVO;
import cn.iocoder.yudao.module.product.controller.admin.harvestbatch.vo.OperateDetailSaveReqVO;
import cn.iocoder.yudao.module.product.controller.admin.landjob.vo.LandJobRespVO;
import cn.iocoder.yudao.module.product.controller.admin.landjobinputs.vo.LandJobInputsRespVO;
import cn.iocoder.yudao.module.product.controller.admin.landplan.vo.*;
import cn.iocoder.yudao.module.product.convert.landplan.LandPlanConvert;
import cn.iocoder.yudao.module.product.dal.dataobject.landplan.LandPlanDO;
import cn.iocoder.yudao.module.product.dal.mysql.landplan.LandPlanMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.product.enums.ErrorCodeConstants.LAND_PLAN_NOT_EXISTS;

/**
 * 地块作业计划 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class LandPlanServiceImpl implements LandPlanService {

    @Resource
    private LandPlanMapper landPlanMapper;

    /**
     * 创建地块作业计划
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    @Override
    public Long createLandPlan(LandPlanCreateReqVO createReqVO) {
        // 插入
        LandPlanDO landPlan = LandPlanConvert.INSTANCE.convert(createReqVO);
        landPlan.setLandPlanId(CodeEnum.LAND_PLAN_CODE.getCode());
        landPlan.setId(null);
        landPlanMapper.insert(landPlan);
        // 返回
        return landPlan.getId();
    }

    /**
     * 批量创建地块作业计划
     *
     * @param createReqVOList 创建信息列表
     * @return 编号列表
     */
    @Override
    public List<Long> createLandPlanList(List<LandPlanCreateReqVO> createReqVOList) {
        List<Long> idList = new ArrayList<>();
        for (LandPlanCreateReqVO createReqVO : createReqVOList) {
            // 插入
            LandPlanDO landPlan = LandPlanConvert.INSTANCE.convert(createReqVO);
            landPlan.setLandPlanId(CodeEnum.LAND_PLAN_CODE.getCode());
            landPlan.setGoodsName(createReqVO.getGoodsName());
            landPlanMapper.insert(landPlan);
            // 添加到编号列表
            idList.add(landPlan.getId());
        }
        // 返回编号列表
        return idList;
    }

    /**
     * 更新地块作业计划
     *
     * @param updateReqVO 更新信息
     */
    @Override
    public void updateLandPlan(LandPlanUpdateReqVO updateReqVO) {
        // 校验存在
        validateLandPlanExists(updateReqVO.getId());
        // 更新
        LandPlanDO updateObj = LandPlanConvert.INSTANCE.convert(updateReqVO);
        landPlanMapper.updateById(updateObj);
    }

    /**
     * 删除地块作业计划
     *
     * @param id 编号
     */
    @Override
    public void deleteLandPlan(Long id) {
        // 校验存在
        validateLandPlanExists(id);
        // 删除
        landPlanMapper.deleteById(id);
    }

    private void validateLandPlanExists(Long id) {
        if (landPlanMapper.selectById(id) == null) {
            throw exception(LAND_PLAN_NOT_EXISTS);
        }
    }

    /**
     * 获得地块作业计划
     *
     * @param id 编号
     * @return 地块作业计划
     */
    @Override
    public LandPlanDO getLandPlan(Long id) {
        return landPlanMapper.selectById(id);
    }

    /**
     * 获得地块作业计划列表
     *
     * @param goodsId 商品编号
     * @return 地块作业计划列表
     */
    @Override
    public List<LandPlanDO> getLandPlanByGoodsId(String goodsId) {
        return landPlanMapper.selectList(new LambdaQueryWrapperX<LandPlanDO>().eq(LandPlanDO::getGoodsId, goodsId));
    }

    /**
     * 获得地块作业计划分页
     *
     * @param pageReqVO 分页查询
     * @return 地块作业计划分页
     */
    @Override
    public PageResult<LandPlanDO> getLandPlanPage(LandPlanPageReqVO pageReqVO) {
        return landPlanMapper.selectPage(pageReqVO);
    }

    @Override
    public List<FarmingPlantReqVO> getLandPlanForFarmingConfig(String goodsId, List<Long> ids, List<String> planIds, Integer blockState) {
        if((ids == null || ids.isEmpty()) && (planIds == null || planIds.isEmpty())) {
            // 直接返回空集合
            return new ArrayList<>();
        }
        List<FarmingPlantReqVO> results = new ArrayList<>();
        String dataFormat = "yyyy-MM-dd";
        Map<String, Object> params = new HashMap<>();
        params.put("goodsId", goodsId);
        params.put("ids", ids);
        params.put("planIds", planIds);
        // params.put("blockState", blockState);
        List<LandPlanRespVO> plans = landPlanMapper.queryLandPlansByParams(params);
        plans.forEach(plan -> {
            FarmingPlantReqVO farmingPlant = new FarmingPlantReqVO();
            farmingPlant.setLinkName("地块名称:" + plan.getLandName());
            farmingPlant.setOperateObjId(plan.getId());

            List<OperateDetailSaveReqVO> planDetails = new ArrayList<>();
            OperateDetailSaveReqVO landIdField = new OperateDetailSaveReqVO();
            landIdField.setTitle("地块编码");
            landIdField.setContent(plan.getLandId());
            landIdField.setType(OperateDetailTypeEnum.TEXT.getType());
            planDetails.add(landIdField);

            OperateDetailSaveReqVO landPlanIdField = new OperateDetailSaveReqVO();
            landPlanIdField.setTitle("计划编号");
            landPlanIdField.setContent(plan.getLandPlanId());
            landPlanIdField.setType(OperateDetailTypeEnum.TEXT.getType());
            planDetails.add(landPlanIdField);

            OperateDetailSaveReqVO planStartTimeField = new OperateDetailSaveReqVO();
            planStartTimeField.setTitle("种植时间");
            planStartTimeField.setContent(DateUtil.format(plan.getPlanStartTime(), dataFormat));
            planStartTimeField.setType(OperateDetailTypeEnum.TEXT.getType());
            planDetails.add(planStartTimeField);

            OperateDetailSaveReqVO planEndTimeField = new OperateDetailSaveReqVO();
            planEndTimeField.setTitle("采收时间");
            planEndTimeField.setContent(DateUtil.format(plan.getPlanEndTime(), dataFormat));
            planEndTimeField.setType(OperateDetailTypeEnum.TEXT.getType());
            planDetails.add(planEndTimeField);

            OperateDetailSaveReqVO planRemarksField = new OperateDetailSaveReqVO();
            planRemarksField.setTitle("说明");
            planRemarksField.setContent(plan.getRemark());
            planRemarksField.setType(OperateDetailTypeEnum.TEXT.getType());
            planDetails.add(planRemarksField);

            List<BlockCertificateSaveReqVO> blockCertificates = new ArrayList<>();
            plan.getLandJobs().forEach(job -> {
                if(job.getLandJobInputs().isEmpty()) {
                    BlockCertificateSaveReqVO jobBlockCertificate = new BlockCertificateSaveReqVO();
                    jobBlockCertificate.setLinkName("农事作业");// 这里写死
                    jobBlockCertificate.setOperateObjId(job.getId());
                    jobBlockCertificate.setOperateType(OperateTypeEnum.JOB.getType());

                    this.setLandJobs(jobBlockCertificate, blockCertificates, job, null, dataFormat);
                } else {
                    // 循环投入品
                    job.getLandJobInputs().forEach(input -> {
                        BlockCertificateSaveReqVO jobBlockCertificate = new BlockCertificateSaveReqVO();
                        jobBlockCertificate.setLinkName("农事作业投入品");// 这里写死
                        jobBlockCertificate.setOperateObjId(input.getId());
                        jobBlockCertificate.setOperateType(OperateTypeEnum.JOB_INPUT.getType());

                        this.setLandJobs(jobBlockCertificate, blockCertificates, job, input, dataFormat);
                    });
                }
            });
            farmingPlant.setOperateDetails(planDetails);
            farmingPlant.setFarmingJobs(blockCertificates);

            results.add(farmingPlant);
        });

        return results;
    }

    private void setLandJobs(BlockCertificateSaveReqVO jobBlockCertificate, List<BlockCertificateSaveReqVO> blockCertificates, LandJobRespVO landJob,
            LandJobInputsRespVO input, String dataFormat) {
        List<OperateDetailSaveReqVO> jobDetails = new ArrayList<>();

        OperateDetailSaveReqVO jobTypeField = new OperateDetailSaveReqVO();
        jobTypeField.setTitle("任务类型");
        jobTypeField.setContent(landJob.getJobTypeName());
        jobTypeField.setType(OperateDetailTypeEnum.TEXT.getType());
        jobDetails.add(jobTypeField);

        OperateDetailSaveReqVO inputNameField = new OperateDetailSaveReqVO();
        inputNameField.setTitle("投入品名称");
        inputNameField.setContent(input == null ? "-" : input.getInputsName());
        inputNameField.setType(OperateDetailTypeEnum.TEXT.getType());
        jobDetails.add(inputNameField);

        OperateDetailSaveReqVO batchIdField = new OperateDetailSaveReqVO();
        batchIdField.setTitle("投入品批次编号");
        batchIdField.setContent(input == null ? "-" : input.getInputsBatchId());
        batchIdField.setType(OperateDetailTypeEnum.TEXT.getType());
        jobDetails.add(batchIdField);

        OperateDetailSaveReqVO jobDateField = new OperateDetailSaveReqVO();
        jobDateField.setTitle("作业日期");
        jobDateField.setContent(DateUtil.format(landJob.getJobDate(), dataFormat));
        jobDateField.setType(OperateDetailTypeEnum.TEXT.getType());
        jobDetails.add(jobDateField);

        OperateDetailSaveReqVO jobWorkerField = new OperateDetailSaveReqVO();
        jobWorkerField.setTitle("作业员工");
        jobWorkerField.setContent(landJob.getJobWorker());
        jobWorkerField.setType(OperateDetailTypeEnum.TEXT.getType());
        jobDetails.add(jobWorkerField);

        OperateDetailSaveReqVO picturesField = new OperateDetailSaveReqVO();
        picturesField.setTitle("作业图片");
        picturesField.setContent(input == null ? landJob.getJobImgUrl() : input.getInputsImgUrl());
        picturesField.setType(OperateDetailTypeEnum.IMAGE.getType());
        jobDetails.add(picturesField);

        OperateDetailSaveReqVO videoField = new OperateDetailSaveReqVO();
        videoField.setTitle("作业视频");
        videoField.setContent(input == null ? landJob.getJobVideoUrl() : input.getInputsVideoUrl());
        videoField.setType(OperateDetailTypeEnum.VIDEO.getType());
        jobDetails.add(videoField);

        OperateDetailSaveReqVO remarkField = new OperateDetailSaveReqVO();
        remarkField.setTitle("说明");
        remarkField.setContent(input != null ? input.getInputsDesc() : landJob.getRemarks());
        remarkField.setType(OperateDetailTypeEnum.TEXT.getType());
        jobDetails.add(remarkField);

        jobBlockCertificate.setOperateDetails(jobDetails);
        blockCertificates.add(jobBlockCertificate);
    }

}
